/* LCD-Display 32×8 mit 4x MAX7219

Hardware: ATmegaX8
Pin	Port		Funktion
1	PC6	(RESET)	LCD R/W
2	PD0	(RxD)	LCD D4
3	PD1	(TxD)	LCD D5
4	PD2		LCD D6
5	PD3		LCD D7
6	PD4	
7	Ucc		Lb
8	GND		b
9	PB6	TOSC1	32-KHz-Quarz
10	PB7	TOSC2	32-KHz-Quarz
11	PD5	OC0B	PWM für Tongenerator
12	PD6	OC0A	PWM für Inverswandler
13	PD7	(AIN1)
14	PB0	ICP1
15	PB1	(OC1A)
16	PB2	(OC1B)
17	PB3	(MOSI)
18	PB4	(MISO)
19	PB5	(SCK)
20	AUcc		Lb
21	AREF
22	GND		b
23	PC0	ADC0	Betriebsspannungsmessung (1/4 Ucc)
24	PC1	ADC1	Überwachung La und CLIP
25	PC2
26	PC3
27	PC4	(ADC4)	LCD RS
28	PC5	(ADC5)	LCD E
*/

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
//#include <avr/sleep.h>
//#include <avr/fuse.h>
#include <avr/signature.h>	// gepatcht: "const" weg!

FUSES={
 0xE2,	// kein CKDIV8
 0x57,	// RSTDISBL, EESAVE
 0xFF
};

typedef unsigned char byte;
#define NOINIT __attribute__((section(".noinit")))

namespace disp{
enum {W=140,H=32};
inline void setRS(bool data) {if (data) PORTC|=0x10; else PORTC&=~0x10;}
inline void setRW(bool read) {if (read) PORTC|=0x40, DDRD&=0xF0, PORTD|=0x0F; else PORTC&=~0x40, DDRD|=0x0F;}
inline void setE(bool e) {if (e) PORTC|=0x20; else PORTC&=~0x20;}

static void writeNibble(byte b) {
 PORTD = PORTD&0xF0 | b;
 _delay_us(2);
 setE(true);
 _delay_us(0.4);
 setE(false);
}
static void writeByte(byte b) {
 writeNibble(b>>4);		// Hi-Nibble zuerst
 writeNibble(b&0x0F);
}
static byte readNibble() {
 _delay_us(2);
 setE(true);
 _delay_us(0.4);
 byte ret=PIND&0x0F;
 setE(false);
 return ret;
}
static byte readByte() {
 byte ret=readNibble()<<4;	// Hi-Nibble zuerst
 ret |= readNibble();
 return ret;
}
static void waitR() {
 setRS(false);
 setRW(true);
 while (readByte()&0x80);	// warte bis BusyFlag == 0 (72 µs)
// _delay_us(100);
}
static void waitW() {
 waitR();
 setRW(false);
}
static void writeC(byte b) {
 waitW();
 writeByte(b);
}
static void writeD(byte b) {
 waitW();
 setRS(true);
 writeByte(b);
}
static byte readD() {
 waitR();
 setRS(true);
 return readByte();
}

static void init() {
 PORTC = 0x0C;
 DDRC  = 0x70;
 DIDR0 = 0x03;
 PORTD = 0xFF;
 DDRD  = 0x0F;
 _delay_ms(40);
 writeNibble(0x20);
 _delay_us(100);
 writeByte(0x20);	// Function set: 4 bit
// _delay_us(100);
 writeC(0x0F);	// On/Off control
// _delay_us(100);
 writeC(0x01);	// Display Clear
// _delay_ms(10);
 writeC(0x06);	// Entry Mode
}
}
// Synchronisation
ISR(TIMER2_OVF_vect) {
}
 
int main() {
 PORTB = 0xFF;
 ACSR  = 0x80;	// Analogvergleicher aus
 ASSR  = 0x20;	// 32-kHz-Oszillator aktivieren
 TCCR2B= 0x05;	// Timer2 Vorteiler 128: 1-Hz-Interruptfrequenz
 TIMSK2= 0x01;	// Interrupts zur Synchronisation des RC-Oszillators (8 MHz)
 disp::init();
 for(char c='!';;c++) {
  _delay_ms(200);
  disp::writeD(c);
 }
}
